import math

class stack_frame:
    entry = None
    t0 = None
    tm = None
    t1 = None
    cur_child_idx = None

    def __init__(self):
        pass

    def __repr__(self):
        return "stack_frame: {entry: %08X, t0: %s, t1: %s, tm: %s, cur_child_idx: %s}" % (
            self.entry, self.t0, self.t1, self.tm, self.cur_child_idx
        )

class fp:
    binval = 0
    global_prec = 16
    size = 32

    def __init__(self, value=2**16):
        if value > 2**(fp.size - 1 - fp.global_prec) - 1:
            value = 2**(fp.size - 1 - fp.global_prec) - 2**(-fp.global_prec)
        elif value < -2**(fp.size - 1 - fp.global_prec) - 1:
            value = -2**(fp.size - 1 - fp.global_prec)

        self.binval = int(value * 2**fp.global_prec) & (2**fp.size - 1)

    def __repr__(self):
        if self.binval > 2**(fp.size - 1) - 1:
            dec = (self.binval - 2**fp.size) / 2**fp.global_prec
        else:
            dec = self.binval / 2**fp.global_prec
        return "%08X (%f)" % (self.binval, dec)

    def add(left, right):
        retval = fp(0)
        if left.binval == 2**(fp.size - 1) or right.binval == 2**(fp.size - 1):
            retval.binval = 2**(fp.size - 1)
        else:
            retval.binval = (left.binval + right.binval) & (2**fp.size - 1)
        return retval

    def shr(val):
        retval = fp(0)
        if val.binval == 2**(fp.size - 1):
            retval.binval = 2**(fp.size - 1)
        elif val.binval < 2**(fp.size - 1):
            retval.binval = val.binval >> 1
        else:
            retval.binval = (val.binval | 2**fp.size) >> 1
        return retval

    def gt(left, right):
        if (
            (left.binval > 2**(fp.size - 1) - 1) ^
            (right.binval > 2**(fp.size - 1) - 1)
        ):
            return (left.binval <= 2**(fp.size - 1) - 1)
        else:
            return left.binval > right.binval

    def lt(left, right):
        if (
            (left.binval > 2**(fp.size - 1) - 1) ^
            (right.binval > 2**(fp.size - 1) - 1)
        ):
            return (left.binval > 2**(fp.size - 1) - 1)
        else:
            return left.binval < right.binval

    def gte(left, right):
        if (
            (left.binval > 2**(fp.size - 1) - 1) ^
            (right.binval > 2**(fp.size - 1) - 1)
        ):
            return (left.binval <= 2**(fp.size - 1) - 1)
        else:
            return left.binval >= right.binval

    def lte(left, right):
        if (
            (left.binval > 2**(fp.size - 1) - 1) ^
            (right.binval > 2**(fp.size - 1) - 1)
        ):
            return (left.binval > 2**(fp.size - 1) - 1)
        else:
            return left.binval <= right.binval


class vec3:
    x = fp()
    y = fp()
    z = fp()

    def __init__(self, x=None, y=None, z=None):
        if x is not None:
            self.x = fp(x)
        if y is not None:
            self.y = fp(y)
        if z is not None:
            self.z = fp(z)


    def __repr__(self):
        return "[%s | %s | %s]" % (self.x, self.y, self.z)

    def add(left, right):
        retval = vec3()
        retval.x = fp.add(left.x, right.x)
        retval.y = fp.add(left.y, right.y)
        retval.z = fp.add(left.z, right.z)
        return retval

    def shr(val):
        retval = vec3()
        retval.x = fp.shr(val.x)
        retval.y = fp.shr(val.y)
        retval.z = fp.shr(val.z)
        return retval

    def max(self):
        if fp.gte(self.x, self.y) and fp.gte(self.x, self.z):
            return self.x
        elif fp.gte(self.y, self.x) and fp.gte(self.y, self.z):
            return self.y
        else:
            return self.z

    def min(self):
        if fp.lte(self.x, self.y) and fp.lte(self.x, self.z):
            return self.x
        elif fp.lte(self.y, self.x) and fp.lte(self.y, self.z):
            return self.y
        else:
            return self.z
