from datatypes import *
import sys

def get_initial_child_idx(t0, tm):
    child_idx = 0

    if fp.gte(t0.x, t0.y) and fp.gte(t0.x, t0.z):
        child_idx |= (fp.gte(t0.x, tm.y) << 1);
        child_idx |= (fp.gte(t0.x, tm.z) << 2);
    elif fp.gte(t0.y, t0.x) and fp.gte(t0.y, t0.z):
        child_idx |= (fp.gte(t0.y, tm.x) << 0);
        child_idx |= (fp.gte(t0.y, tm.z) << 2);
    else:
        child_idx |= (fp.gte(t0.z, tm.x) << 0);
        child_idx |= (fp.gte(t0.z, tm.y) << 1);

    return child_idx


def get_max_t_value(t):
    if fp.gte(t.x, t.y) and fp.gte(t.x, t.z):
        return t.x
    if fp.gte(t.y, t.x) and fp.gte(t.y, t.z):
        return t.y
    else:
        return t.z


def get_next_child_idx(cur_child_idx, t):
    lookup = [
        [0b0001, 0b0010, 0b0100],
        [0b1000, 0b0011, 0b0101],
        [0b0011, 0b1000, 0b0110],
        [0b1000, 0b1000, 0b0111],
        [0b0101, 0b0110, 0b1000],
        [0b1000, 0b0111, 0b1000],
        [0b0111, 0b1000, 0b1000],
        [0b1000, 0b1000, 0b1000]
    ]

    if fp.lte(t.x, t.y) and fp.lte(t.x, t.z):
        return lookup[cur_child_idx][0]
    if fp.lte(t.y, t.x) and fp.lte(t.y, t.z):
        return lookup[cur_child_idx][1]
    else:
        return lookup[cur_child_idx][2]


def get_child_t_values(child_idx, t0, t1):
    t = vec3()
    t.x = t1.x if (child_idx & 0x1) else t0.x
    t.y = t1.y if (child_idx & 0x2) else t0.y
    t.z = t1.z if (child_idx & 0x4) else t0.z
    return t


def sum_and_right_shift(t0, t1):
    return vec3.shr(vec3.add(t0, t1))

state = None
stack = []
stack_idx = 0
out = {
    'ready': False,
    't_hit': None
}

t0_child = vec3()
t1_child = vec3()
next_child = 0
a = 0

def rt_core(reset, start, params, mem, debug):
    global state
    global stack
    global stack_idx
    global out
    global t0_child
    global t1_child
    global next_child
    global a
    global push_entry
    global push_offset_lookup


    if debug:
        print('=== %s ===' % state)
        print('\tOut: %s\n\tStack: %s' % (out, stack))

    if reset:
        state = 'IDLE'
        out['ready'] = True
        return out

    if state == 'IDLE':
        stack = []
        t0_child = vec3()
        t1_child = vec3()
        next_child = 0
        stack_idx = 0
        if start:
            state = 'INIT'
            out['ready'] = False
            stack.append(stack_frame())
            stack[stack_idx].entry = params['entry']
            stack[stack_idx].t0 = params['t0']
            stack[stack_idx].t1 = params['t1']
            stack[stack_idx].tm = sum_and_right_shift(
                params['t0'],
                params['t1']
            )
            a = params['a']
        else:
            return out
    elif state == 'INIT':
        stack[stack_idx].cur_child_idx = get_initial_child_idx(
            stack[stack_idx].t0,
            stack[stack_idx].tm
        );
        state = 'EVAL'
    elif state == 'EVAL':
        t0_child = get_child_t_values(
            stack[stack_idx].cur_child_idx,
            stack[stack_idx].t0,
            stack[stack_idx].tm
        )
        t1_child = get_child_t_values(
            stack[stack_idx].cur_child_idx,
            stack[stack_idx].tm,
            stack[stack_idx].t1
        )
        if (
            ((mem[stack[stack_idx].entry] & 0x0000FF00) >> 8) &
            (1 << (a ^ stack[stack_idx].cur_child_idx))
        ):
            if (
                (mem[stack[stack_idx].entry] & 0x000000FF) &
                (1 << (a ^ stack[stack_idx].cur_child_idx))
            ):
                out['hit'] = True
                state = 'OUT'
            else:
                state = 'PUSH'
                push_entry = stack[stack_idx].entry + \
                             ((mem[stack[stack_idx].entry] & 0x7FFF0000) >> 16)
                mask = ((mem[stack[stack_idx].entry] & 0x0000FF00) >> 8) & \
                       ~(mem[stack[stack_idx].entry] & 0x000000FF)
                push_offset_lookup = [
                    0,
                    mask & 0x1,
                    (mask & 0x1) + ((mask >> 1) & 0x1),
                    (mask & 0x1) + ((mask >> 1) & 0x1) + ((mask >> 2) & 0x1),
                    (mask & 0x1) + ((mask >> 1) & 0x1) + ((mask >> 2) & 0x1) +
                        ((mask >> 3) & 0x1),
                    (mask & 0x1) + ((mask >> 1) & 0x1) + ((mask >> 2) & 0x1) +
                        ((mask >> 3) & 0x1) + ((mask >> 4) & 0x1),
                    (mask & 0x1) + ((mask >> 1) & 0x1) + ((mask >> 2) & 0x1) +
                        ((mask >> 3) & 0x1) + ((mask >> 4) & 0x1) +
                        ((mask >> 5) & 0x1),
                    (mask & 0x1) + ((mask >> 1) & 0x1) + ((mask >> 2) & 0x1) +
                        ((mask >> 3) & 0x1) + ((mask >> 4) & 0x1) +
                        ((mask >> 5) & 0x1) + ((mask >> 6) & 0x1),
                ]
        else:
            next_child = get_next_child_idx(
                stack[stack_idx].cur_child_idx,
                t1_child
            )
            state = 'NEXT'
    elif state == 'PUSH':
        entry = 0
        if mem[stack[stack_idx].entry] & 0x80000000:
            entry = stack[stack_idx].entry + mem[push_entry] + \
                    push_offset_lookup[a ^ stack[stack_idx].cur_child_idx]
        else:
            entry = push_entry + \
                    push_offset_lookup[a ^ stack[stack_idx].cur_child_idx]

        stack_idx += 1
        stack.append(stack_frame())
        stack[stack_idx].entry = entry
        stack[stack_idx].t0 = t0_child
        stack[stack_idx].t1 = t1_child
        stack[stack_idx].tm = sum_and_right_shift(t0_child, t1_child)
        state = 'INIT'
    elif state == 'NEXT':
        if next_child < 8:
            stack[stack_idx].cur_child_idx = next_child
            state = 'EVAL'
        else:
            if stack_idx == 0:
                out['hit'] = False
                state = 'OUT'
            else:
                state = 'POP'
    elif state == 'POP':
        del stack[stack_idx]
        stack_idx -= 1
        t0_child = get_child_t_values(
            stack[stack_idx].cur_child_idx,
            stack[stack_idx].t0,
            stack[stack_idx].tm
        )
        t1_child = get_child_t_values(
            stack[stack_idx].cur_child_idx,
            stack[stack_idx].tm,
            stack[stack_idx].t1
        )
        next_child = get_next_child_idx(
            stack[stack_idx].cur_child_idx,
            t1_child
        )
        state = 'NEXT'
    elif state == 'OUT':
        out['t_hit'] = get_max_t_value(t0_child)
        if out['t_hit'] == t0_child.x:
            out['normal'] = vec3(1, 0, 0)
        elif out['t_hit'] == t0_child.y:
            out['normal'] = vec3(0, 1, 0)
        else:
            out['normal'] = vec3(0, 0, 1)
        state = 'IDLE'
        out['ready'] = True

    return out


