import types::*;

module rtu_v1_0 #(
    parameter integer C_S00_AXIS_TDATA_WIDTH = 256,
    parameter integer C_M00_AXIS_TDATA_WIDTH = 256,
    parameter integer CORE_CNT = 4,
    parameter integer FX_PT_FRAC_BITS = 24
)(
    // Ports of Axi Slave Bus Interface S00_AXIS
    input  wire  s00_axis_aclk,
    input  wire  s00_axis_aresetn,
    output wire  s00_axis_tready,
    input  wire [C_S00_AXIS_TDATA_WIDTH-1 : 0] s00_axis_tdata,
    input  wire [(C_S00_AXIS_TDATA_WIDTH/8)-1 : 0] s00_axis_tstrb,
    input  wire  s00_axis_tlast,
    input  wire  s00_axis_tvalid,

    // Ports of Axi Master Bus Interface M00_AXIS
    input  wire  m00_axis_aclk,
    input  wire  m00_axis_aresetn,
    output wire  m00_axis_tvalid,
    output wire [C_M00_AXIS_TDATA_WIDTH-1 : 0] m00_axis_tdata,
    output wire [(C_M00_AXIS_TDATA_WIDTH/8)-1 : 0] m00_axis_tstrb,
    output wire  m00_axis_tlast,
    input  wire  m00_axis_tready
);

    wire         job_ready;
    wire         job_valid;
    wire [255:0] job_data;

    wire  [31:0] job_count;
    wire         in_done;
    wire         out_done;

    wire         result_ready;
    wire         result_valid;
    wire [255:0] result_data;

    // Instantiation of Axi Bus Interface S00_AXIS
    rtu_v1_0_S00_AXIS #(
        .C_S_AXIS_TDATA_WIDTH(C_S00_AXIS_TDATA_WIDTH)
    ) rtu_v1_0_S00_AXIS_inst (
        .job_ready(job_ready),
        .job_valid(job_valid),
        .job_data(job_data),
        .job_count(job_count),
        .in_done(in_done),
        .out_done(out_done),

        .S_AXIS_ACLK(s00_axis_aclk),
        .S_AXIS_ARESETN(s00_axis_aresetn),
        .S_AXIS_TREADY(s00_axis_tready),
        .S_AXIS_TDATA(s00_axis_tdata),
        .S_AXIS_TSTRB(s00_axis_tstrb),
        .S_AXIS_TLAST(s00_axis_tlast),
        .S_AXIS_TVALID(s00_axis_tvalid)
    );

    // Instantiation of Axi Bus Interface M00_AXIS
    rtu_v1_0_M00_AXIS #(
        .C_M_AXIS_TDATA_WIDTH(C_M00_AXIS_TDATA_WIDTH)
    ) rtu_v1_0_M00_AXIS_inst (
        .result_ready(result_ready),
        .result_valid(result_valid),
        .result_data(result_data),
        .job_count(job_count),
        .in_done(in_done),
        .out_done(out_done),

        .M_AXIS_ACLK(m00_axis_aclk),
        .M_AXIS_ARESETN(m00_axis_aresetn),
        .M_AXIS_TVALID(m00_axis_tvalid),
        .M_AXIS_TDATA(m00_axis_tdata),
        .M_AXIS_TSTRB(m00_axis_tstrb),
        .M_AXIS_TLAST(m00_axis_tlast),
        .M_AXIS_TREADY(m00_axis_tready)
    );

    wire     clk;
    wire     reset;
    wire     in_ready;
    wire     in_valid;
    ray_t    in_ray;
    wire     out_ready;
    wire     out_valid;
    result_t out_result;

    rtu #(
        .CORE_CNT(CORE_CNT),
        .FX_PT_FRAC_BITS(FX_PT_FRAC_BITS)
    ) rtu_inst (
        .clk(clk),
        .reset(reset),

        .in_ready(in_ready),
        .in_valid(in_valid),
        .in_ray(in_ray),

        .out_ready(out_ready),
        .out_valid(out_valid),
        .out_result(out_result)
    );

    assign clk          =  s00_axis_aclk;
    assign reset        = ~s00_axis_aresetn;

    assign job_ready     = in_ready;
    assign in_valid      = job_valid;

    assign in_ray.job_id = job_data[0  +:32];
    assign in_ray.orig   = job_data[32 +:96];
    assign in_ray.dir    = job_data[128+:96];
    assign in_ray.addr   = 'b0;

    assign out_ready    = result_ready;
    assign result_valid = out_valid;
    assign result_data  = {
        32'b0,
        out_result.cost,
        out_result.normal,
        out_result.t_hit,
        31'b0, out_result.hit,
        out_result.job_id
    };
endmodule
