module rtu_v1_0_M00_AXIS #(
    parameter integer C_M_AXIS_TDATA_WIDTH  = 256
)(
    output reg          result_ready,
    input  wire         result_valid,
    input  wire [255:0] result_data,
    input  wire  [31:0] job_count,
    input  wire         in_done,
    output reg          out_done,

    // Do not modify the ports beyond this line
    input  wire  M_AXIS_ACLK,
    input  wire  M_AXIS_ARESETN,
    output wire  M_AXIS_TVALID,
    output wire [C_M_AXIS_TDATA_WIDTH-1 : 0] M_AXIS_TDATA,
    output wire [(C_M_AXIS_TDATA_WIDTH/8)-1 : 0] M_AXIS_TSTRB,
    output wire  M_AXIS_TLAST,
    input  wire  M_AXIS_TREADY
);
    wire        axi_ready;
    wire        axi_valid;
    reg         axi_last;
    reg [255:0] axi_data;
    reg  [31:0] result_count;

    assign M_AXIS_TVALID = axi_valid;
    assign M_AXIS_TDATA  = axi_data;
    assign M_AXIS_TLAST  = axi_last;
    assign M_AXIS_TSTRB  = {(C_M_AXIS_TDATA_WIDTH/8){1'b1}};
    assign axi_ready     = M_AXIS_TREADY;
    assign axi_valid     = ~result_ready;

    always @(posedge M_AXIS_ACLK) begin
        if (!M_AXIS_ARESETN) begin
            axi_last     <= 'b0;
            axi_data     <= 'b0;
            result_count <= 'b0;
            result_ready <= 'b1;
            out_done     <= 'b0;
        end else begin
            if (result_ready && result_valid) begin
                axi_data     <= result_data;
                result_count <= result_count + 1;
                axi_last     <= in_done & (result_count + 1 == job_count);
                out_done     <= in_done & (result_count + 1 == job_count);
                result_ready <= 'b0;
            end

            if (axi_ready & axi_valid) begin
                result_ready <= 'b1;
                axi_last     <= 'b0;
                out_done     <= 'b0;
            end
        end
    end
endmodule
