module rtu_v1_0_S00_AXIS #(
    parameter integer C_S_AXIS_TDATA_WIDTH  = 256
)(
    input  wire        job_ready,
    input  wire        out_done,
    output reg         job_valid,
    output reg [255:0] job_data,

    output reg  [31:0] job_count,
    output reg         in_done,

    // Do not modify the ports beyond this line
    input  wire  S_AXIS_ACLK,
    input  wire  S_AXIS_ARESETN,
    output wire  S_AXIS_TREADY,
    input  wire [C_S_AXIS_TDATA_WIDTH-1 : 0] S_AXIS_TDATA,
    input  wire [(C_S_AXIS_TDATA_WIDTH/8)-1 : 0] S_AXIS_TSTRB,
    input  wire  S_AXIS_TLAST,
    input  wire  S_AXIS_TVALID
);
    reg          axi_ready;
    wire         axi_valid;
    wire         axi_last;
    wire [255:0] axi_data;

    assign S_AXIS_TREADY = axi_ready;
    assign axi_valid     = S_AXIS_TVALID;
    assign axi_data      = S_AXIS_TDATA;
    assign axi_last      = S_AXIS_TLAST;

    always @(posedge S_AXIS_ACLK) begin
        if (!S_AXIS_ARESETN) begin
            axi_ready <= 'b1;
            job_valid <= 'b0;
            job_data  <= 'b0;
            job_count <= 'b0;
            in_done   <= 'b0;
        end else begin
            if (axi_ready && axi_valid) begin
                if (axi_last) begin
                    in_done <= 'b1;
                end

                job_data  <= axi_data;
                axi_ready <= 'b0;
                job_valid <= 'b1;
                job_count <= job_count + 1;
            end

            if (job_ready && job_valid) begin
                axi_ready <= 'b1;
                job_valid <= 'b0;
                job_data  <= 'b0;
            end

            if (in_done && out_done) begin
                in_done <= 'b0;
            end
        end
    end
endmodule
