import types::*;

module float_to_fixed #(
    parameter FX_PT_FRAC_BITS = 16
) (
    input   logic        clk,
    input   logic        enable,
    input   float32      data_in,
    output  fixed32      data_out
);

typedef struct packed {
    logic        sign;
    logic        exp_sign;
    logic [7:0]  exponent;
    logic [63:0] fraction;
} pipeline_stage_t;

pipeline_stage_t s1;
pipeline_stage_t s2;
pipeline_stage_t s3;
pipeline_stage_t s4;

logic [31:0] output_select;
assign output_select = s4.fraction[(63-FX_PT_FRAC_BITS)-:32];

always @(posedge clk) begin
    if (enable) begin
        s1.exp_sign <= 'b0;
        s1.sign     <= data_in[31];
        s1.fraction <= 'b0;
        s1.fraction[32-:24] <= {1'b1, data_in[22:0]};

        if ($signed(data_in[30:23] - 127) >= 0) begin
            s1.exponent <= data_in[30:23] - 127;
        end else begin
            s1.exponent <= ~(data_in[30:23] - 127) + 1;
            s1.exp_sign <= 'b1;
        end

        if ($signed(data_in[30:23] - 127) >= (31 - FX_PT_FRAC_BITS)) begin
            s1.exponent <= 'b0;
            s1.sign     <= 'b0;
            if (data_in[31]) begin
                s1.fraction <= 'b0;
                s1.fraction[63] <= 'b1;
            end else begin
                s1.fraction <= 'b1;
                s1.fraction[63] <= 'b0;
            end
        end else if (
            $signed(data_in[30:23] - 127) <= $signed(1 - FX_PT_FRAC_BITS)
        ) begin
            s1.sign         <= 'b0;
            s1.exponent     <= 'b0;
            s1.fraction     <= 'b0;
        end
    end
end

always @(posedge clk) begin
    if (enable) begin
        s2 <= s1;

        if (s1.exponent[3]) begin
            if (s1.exp_sign) begin
                s2.fraction <= s1.fraction >> 8;
            end else begin
                s2.fraction <= s1.fraction << 8;
            end
        end
    end
end

always @(posedge clk) begin
    if (enable) begin
        s3 <= s2;

        if (s2.exponent[2]) begin
            if (s2.exp_sign) begin
                s3.fraction <= s2.fraction >> 4;
            end else begin
                s3.fraction <= s2.fraction << 4;
            end
        end
    end
end

always @(posedge clk) begin
    if (enable) begin
        s4 <= s3;

        if (s3.exponent[1]) begin
            if (s3.exp_sign) begin
                s4.fraction <= s3.fraction >> 2;
            end else begin
                s4.fraction <= s3.fraction << 2;
            end
        end
    end
end

always @(posedge clk) begin
    if (enable) begin
        data_out <= s4.sign ? (~output_select + 1) : output_select;

        if (s4.exponent[0]) begin
            if (s4.exp_sign) begin
                data_out <= (
                    s4.sign ? (~(output_select >> 1) + 1) : (output_select >> 1)
                );
            end else begin
                data_out <= (
                    s4.sign ? (~(output_select << 1) + 1) : (output_select << 1)
                );
            end
        end
    end
end
endmodule

