import types::*;

module job_manager #(
    parameter FX_PT_FRAC_BITS = 24
) (
    input  logic        clk,
    input  logic        reset,

    output logic        in_ready,
    input  logic        in_valid,
    input  ray_t        in_ray,

    input  logic        out_ready,
    output logic        out_valid,
    output job_t        out_job
);

logic pipeline_enable;
assign pipeline_enable = out_ready || ~out_valid;
assign in_ready = pipeline_enable;

vec3_t t0_float;
vec3_t t1_float;

genvar i;
generate
    for (i = 0; i < 3; i += 1) begin
        ray_to_params ray_to_params(
            .clk(clk),
            .enable(pipeline_enable),
            .in_ray_orig(in_ray.orig[i*32+:32]),
            .in_ray_dir(in_ray.dir[i*32+:32]),
            .out_t0(t0_float[i*32+:32]),
            .out_t1(t1_float[i*32+:32])
        );
        float_to_fixed #(FX_PT_FRAC_BITS) float_to_fixed_t0(
            .clk(clk),
            .enable(pipeline_enable),
            .data_in(t0_float[i*32+:32]),
            .data_out(out_job.t0[i*32+:32])
        );
        float_to_fixed #(FX_PT_FRAC_BITS) float_to_fixed_t1(
            .clk(clk),
            .enable(pipeline_enable),
            .data_in(t1_float[i*32+:32]),
            .data_out(out_job.t1[i*32+:32])
        );
    end
endgenerate

`define P_STAGES 6

logic [`P_STAGES - 1:0]  valid_delay;
job_t pipeline [`P_STAGES];

assign out_valid      = valid_delay[`P_STAGES - 1];
assign out_job.a      = pipeline[`P_STAGES - 1].a;
assign out_job.addr   = pipeline[`P_STAGES - 1].addr;
assign out_job.job_id = pipeline[`P_STAGES - 1].job_id;

always @(posedge clk) begin
    if (reset == 'b1) begin
        valid_delay <= 'b0;
    end else if (pipeline_enable) begin
        valid_delay        <= {valid_delay[`P_STAGES - 2:0], in_valid & in_ready};
        pipeline[0].a      <= in_ray.job_id[30:28];
        pipeline[0].addr   <= in_ray.addr;
        pipeline[0].job_id <= in_ray.job_id;

        for (int i = 0; i < `P_STAGES - 1; i += 1) begin
            pipeline[i + 1] <= pipeline[i];
        end
    end
end
endmodule
