import types::*;

module result_manager #(
    parameter CORE_CNT = 32
) (
    input  logic                   clk,
    input  logic                   reset,

    output logic    [CORE_CNT-1:0] core_ready,
    input  logic    [CORE_CNT-1:0] core_valid,
    input  result_t [CORE_CNT-1:0] core_result,

    input  logic                   out_ready,
    output logic                   out_valid,
    output result_t                out_result
);

logic    [$clog2(CORE_CNT)-1:0] result_select;
logic    [CORE_CNT-1:0] result_valid;
result_t [CORE_CNT-1:0] result_buffer;

assign out_valid  = result_valid[result_select];
assign core_ready = ~result_valid;
assign out_result = result_buffer[result_select];

always @(posedge clk) begin
    if (reset == 1'b1) begin
        result_valid     <= 'b0;
        result_select    <= 'b0;
    end else begin
        for (int i = 0; i < CORE_CNT; i += 1) begin
            if (core_ready[i] & core_valid[i]) begin
                result_valid[i]  <= 'b1;
                result_buffer[i] <= core_result[i];
            end
        end

        if (out_ready & out_valid) begin
            result_valid[result_select] <= 'b0;
        end

        result_select <= 'b0;

        if (result_select < CORE_CNT - 1) begin
            result_select <= result_select + 1;
        end
    end
end
endmodule
