import types::*;

module rtu #(
    parameter CORE_CNT = 4,
    parameter FX_PT_FRAC_BITS = 24
) (
    input  logic        clk,
    input  logic        reset,

    output logic        in_ready,
    input  logic        in_valid,
    input  ray_t        in_ray,

    input  logic        out_ready,
    output logic        out_valid,
    output result_t     out_result
);

logic sched_ready;
logic sched_valid;
job_t pending_job;

job_manager #(FX_PT_FRAC_BITS) job_manager_inst (
    .clk(clk),
    .reset(reset),

    .in_ready(in_ready),
    .in_valid(in_valid),
    .in_ray(in_ray),

    .out_ready(sched_ready),
    .out_valid(sched_valid),
    .out_job(pending_job)
);


logic [CORE_CNT-1:0] core_in_ready;
logic [CORE_CNT-1:0] core_in_valid;

scheduler #(CORE_CNT) scheduler_inst (
    .clk(clk),
    .reset(reset),

    .in_valid(sched_valid),
    .in_ready(sched_ready),

    .core_ready(core_in_ready),
    .core_valid(core_in_valid)
);

logic [31:0] mem_addr [0:CORE_CNT-1];
logic [31:0] mem_data [0:CORE_CNT-1];
memory #(CORE_CNT) memory_inst(
    .addr(mem_addr),
    .data(mem_data)
);


logic    [CORE_CNT-1:0] core_out_ready;
logic    [CORE_CNT-1:0] core_out_valid;
result_t [CORE_CNT-1:0] core_out_result;

genvar i;
generate
    for (i = 0; i < CORE_CNT; i += 1) begin
        svo_trav_core svo_trav_core_inst(
            .clk(clk),
            .reset(reset),
            .mem_addr(mem_addr[i]),
            .mem_data(mem_data[i]),

            .in_ready(core_in_ready[i]),
            .in_valid(core_in_valid[i]),
            .in_job(pending_job),

            .out_ready(core_out_ready[i]),
            .out_valid(core_out_valid[i]),
            .out_result(core_out_result[i])
        );

    end
endgenerate

result_manager #(CORE_CNT) result_manager_inst (
    .clk(clk),
    .reset(reset),

    .core_ready(core_out_ready),
    .core_valid(core_out_valid),
    .core_result(core_out_result),

    .out_ready(out_ready),
    .out_valid(out_valid),
    .out_result(out_result)
);

endmodule
